@@ -4,34 +4,25 @@
 #define ACODEEXTENSION ".a3c"
 
 /* Basic types */
-#include <limits.h>
 #include <stddef.h>
 #include <stdint.h>
 
-typedef uint32_t Aptr;       /* Type for an ACODE pointer used in the structures */
-/* TODO: Here's the major 32->64bit problem: Aptrs are 32 bit to fit
-   into the 32-bit structure of the Amachine, but sometimes this is
-   used to store a *real* pointer value, which on 64-bit machines are
-   64bits. So for now we'll compile for 32bits to achieve complete
-   cross-platform compatibility of game files.
+typedef uint32_t Aptr;       /* Type for a mapped possibly 64-bit pointer used in the structures */
+/* Aptrs are 32 bit to fit into the 32-bit structure of the Amachine,
+   but represents a *real* pointer value, which on 64-bit machines are
+   64bits. So an Aptr is a symbolic value for the actual address and
+   has to be translated through fromAptr() in memory.c
  */
 
 typedef uint32_t Aword;      /* Type for an ACODE word */
 typedef uint32_t Aaddr;      /* Type for an ACODE address in Amemory */
 typedef uint32_t Aid;        /* Type for an ACODE Instance Id value */
-typedef int32_t Abool;       /* Type for an ACODE Boolean value */
+typedef uint32_t Abool;      /* Type for an ACODE Boolean value */
+typedef uint32_t Aset;       /* Type for an ACODE Set value */
 typedef int32_t Aint;        /* Type for an ACODE Integer value */
-typedef int32_t Aset;        /* Type for an ACODE Set value */
 typedef int32_t CodeValue;   /* Definition for the packing process */
 
 
-#ifndef TRUE
-#define TRUE (0==0)
-#endif
-#ifndef FALSE
-#define FALSE (!TRUE)
-#endif
-
 /* Constants for the Acode file, words/block & bytes/block */
 #define BLOCKLEN 256L
 #define BLOCKSIZE (BLOCKLEN*sizeof(Aword))
@@ -484,20 +475,77 @@ typedef struct IfidEntry {      /* IFIDs are (name, value) pairs */
 /* AMACHINE Header */
 
 typedef struct ACodeHeader {
+    /* Important info */
+    char tag[4];                /* "ALAN" */
+    char version[4];            /* Version of compiler */
+    Aword uid;                  /* Unique id of the compiled game */
+    Aword size;                 /* Size of Acode-part of the file in Awords (strings are stored after) */
+    /* Options */
+    Abool pack;                  /* Is the text packed and encoded ? */
+    Aword stringOffset;          /* Offset to string data in game file */
+    Aword pageLength;            /* Length of a displayed page */
+    Aword pageWidth;             /* and width */
+    Aword debug;                 /* Option: debug? */
+    /* Data structures */
+    Aaddr classTableAddress;
+    Aword classMax;             /* 10 */
+    Aword entityClassId;
+    Aword thingClassId;
+    Aword objectClassId;
+    Aword locationClassId;
+    Aword actorClassId;
+    Aword literalClassId;
+    Aword integerClassId;
+    Aword stringClassId;
+    Aaddr instanceTableAddress;	/* Instance table */
+    Aword instanceMax;          /* 20 - Highest number of an instance */
+    Aword theHero;              /* The hero instance code (id) */
+    Aaddr containerTableAddress;
+    Aword containerMax;
+    Aaddr scriptTableAddress;
+    Aword scriptMax;
+    Aaddr eventTableAddress;
+    Aword eventMax;
+    Aaddr syntaxTableAddress;
+    Aaddr parameterMapAddress;
+    Aword syntaxMax;            /* 30 */
+    Aaddr dictionary;
+    Aaddr verbTableAddress;
+    Aaddr ruleTableAddress;
+    Aaddr messageTableAddress;
+    /* Miscellaneous */
+    Aint attributesAreaSize;	/* Size of attribute data area in Awords */
+    Aint maxParameters;         /* Maximum number of parameters in any syntax */
+    Aaddr stringInitTable;      /* String init table address */
+    Aaddr setInitTable;         /* Set init table address */
+    Aaddr start;                /* Address to Start code */
+    Aword maximumScore;         /* 40 - Maximum score */
+    Aaddr scores;               /* Score table */
+    Aint scoreCount;            /* Max index into scores table */
+    Aaddr sourceFileTable;      /* Table of fpos/len for source filenames */
+    Aaddr sourceLineTable;      /* Table of available source lines to break on */
+    Aaddr freq;                 /* Address to Char freq's for coding */
+    Aword acdcrc;               /* 46 - Checksum for acd code (excl. hdr) */
+    Aword txtcrc;               /* Checksum for text data file, not used */
+    Aaddr ifids;                /* 48 - Address to IFIDS */
+    Aaddr prompt;
+} ACodeHeader;
+
+typedef struct Pre3_0beta2Header {
     /* Important info */
     char tag[4];              /* "ALAN" */
     char version[4];          /* Version of compiler */
     Aword uid;                /* Unique id of the compiled game */
     Aword size;               /* Size of ACD-file in Awords */
     /* Options */
-    Abool pack;               /* Is the text packed and encoded ? */
+    Abool pack;               /* Is the text packed ? */
     Aword stringOffset;       /* Offset to string data in game file */
-    Aword pageLength;         /* Length of a displayed page */
+    Aword pageLength;         /* Length of a page */
     Aword pageWidth;          /* and width */
     Aword debug;              /* Option: debug */
     /* Data structures */
-    Aaddr classTableAddress;
-    Aword classMax;
+    Aaddr classTableAddress;  /* Class table */
+    Aword classMax;           /* Number of classes */
     Aword entityClassId;
     Aword thingClassId;
     Aword objectClassId;
@@ -523,94 +571,37 @@ typedef struct ACodeHeader {
     Aaddr ruleTableAddress;
     Aaddr messageTableAddress;
     /* Miscellaneous */
-    Aint attributesAreaSize;	/* Size of attribute data area in Awords */
-    Aint maxParameters;		/* Maximum number of parameters in any syntax */
-    Aaddr stringInitTable;	/* String init table address */
-    Aaddr setInitTable;		/* Set init table address */
-    Aaddr start;		/* Address to Start code */
-    Aword maximumScore;		/* Maximum score */
-    Aaddr scores;		/* Score table */
-    Aint scoreCount;		/* Max index into scores table */
-    Aaddr sourceFileTable;	/* Table of fpos/len for source filenames */
-    Aaddr sourceLineTable;	/* Table of available source lines to break on */
-    Aaddr freq;			/* Address to Char freq's for coding */
-    Aword acdcrc;		/* Checksum for acd code (excl. hdr) */
-    Aword txtcrc;		/* Checksum for text data file */
-    Aaddr ifids;		/* Address to IFIDS */
-    Aaddr prompt;
-} ACodeHeader;
-
-typedef struct Pre3_0beta2Header {
-    /* Important info */
-    char tag[4];		/* "ALAN" */
-    char version[4];		/* Version of compiler */
-    Aword uid;			/* Unique id of the compiled game */
-    Aword size;			/* Size of ACD-file in Awords */
-    /* Options */
-    Abool pack;			/* Is the text packed ? */
-    Aword stringOffset;		/* Offset to string data in game file */
-    Aword pageLength;		/* Length of a page */
-    Aword pageWidth;		/* and width */
-    Aword debug;		/* Option: debug */
-    /* Data structures */
-    Aaddr classTableAddress;	/* Class table */
-    Aword classMax;		/* Number of classes */
-    Aword entityClassId;
-    Aword thingClassId;
-    Aword objectClassId;
-    Aword locationClassId;
-    Aword actorClassId;
-    Aword literalClassId;
-    Aword integerClassId;
-    Aword stringClassId;
-    Aaddr instanceTableAddress;	/* Instance table */
-    Aword instanceMax;		/* Highest number of an instance */
-    Aword theHero;		/* The hero instance code (id) */
-    Aaddr containerTableAddress;
-    Aword containerMax;
-    Aaddr scriptTableAddress;
-    Aword scriptMax;
-    Aaddr eventTableAddress;
-    Aword eventMax;
-    Aaddr syntaxTableAddress;
-    Aaddr parameterMapAddress;
-    Aword syntaxMax;
-    Aaddr dictionary;
-    Aaddr verbTableAddress;
-    Aaddr ruleTableAddress;
-    Aaddr messageTableAddress;
-    /* Miscellaneous */
-    Aint attributesAreaSize;	/* Size of attribute data area in Awords */
-    Aint maxParameters;		/* Maximum number of parameters in any syntax */
-    Aaddr stringInitTable;	/* String init table address */
-    Aaddr setInitTable;		/* Set init table address */
-    Aaddr start;		/* Address to Start code */
-    Aword maximumScore;		/* Maximum score */
-    Aaddr scores;		/* Score table */
-    Aint scoreCount;		/* Max index into scores table */
-    Aaddr sourceFileTable;	/* Table of fpos/len for source filenames */
-    Aaddr sourceLineTable;	/* Table of available source lines to break on */
-    Aaddr freq;			/* Address to Char freq's for coding */
-    Aword acdcrc;		/* Checksum for acd code (excl. hdr) */
-    Aword txtcrc;		/* Checksum for text data file */
-    Aaddr ifids;		/* Address to IFIDS */
+    Aint attributesAreaSize;    /* Size of attribute data area in Awords */
+    Aint maxParameters;         /* Maximum number of parameters in any syntax */
+    Aaddr stringInitTable;      /* String init table address */
+    Aaddr setInitTable;         /* Set init table address */
+    Aaddr start;                /* Address to Start code */
+    Aword maximumScore;         /* Maximum score */
+    Aaddr scores;               /* Score table */
+    Aint scoreCount;            /* Max index into scores table */
+    Aaddr sourceFileTable;      /* Table of fpos/len for source filenames */
+    Aaddr sourceLineTable;      /* Table of available source lines to break on */
+    Aaddr freq;                 /* Address to Char freq's for coding */
+    Aword acdcrc;               /* Checksum for acd code (excl. hdr) */
+    Aword txtcrc;               /* Checksum for text data file */
+    Aaddr ifids;                /* Address to IFIDS */
 } Pre3_0beta2Header;
 
 typedef struct Pre3_0alpha5Header {
     /* Important info */
-    char tag[4];		/* "ALAN" */
-    char version[4];		/* Version of compiler */
-    Aword uid;			/* Unique id of the compiled game */
-    Aword size;			/* Size of ACD-file in Awords */
+    char tag[4];              /* "ALAN" */
+    char version[4];          /* Version of compiler */
+    Aword uid;                /* Unique id of the compiled game */
+    Aword size;               /* Size of ACD-file in Awords */
     /* Options */
-    Abool pack;			/* Is the text packed ? */
-    Aword stringOffset;		/* Offset to string data in game file */
-    Aword pageLength;		/* Length of a page */
-    Aword pageWidth;		/* and width */
-    Aword debug;		/* Option: debug */
+    Abool pack;               /* Is the text packed ? */
+    Aword stringOffset;       /* Offset to string data in game file */
+    Aword pageLength;         /* Length of a page */
+    Aword pageWidth;          /* and width */
+    Aword debug;              /* Option: debug */
     /* Data structures */
-    Aaddr classTableAddress;	/* Class table */
-    Aword classMax;		/* Number of classes */
+    Aaddr classTableAddress;  /* Class table */
+    Aword classMax;           /* Number of classes */
     Aword entityClassId;
     Aword thingClassId;
     Aword objectClassId;
@@ -620,8 +611,8 @@ typedef struct Pre3_0alpha5Header {
     Aword integerClassId;
     Aword stringClassId;
     Aaddr instanceTableAddress;	/* Instance table */
-    Aword instanceMax;		/* Highest number of an instance */
-    Aword theHero;		/* The hero instance code (id) */
+    Aword instanceMax;          /* Highest number of an instance */
+    Aword theHero;              /* The hero instance code (id) */
     Aaddr containerTableAddress;
     Aword containerMax;
     Aaddr scriptTableAddress;
@@ -636,19 +627,19 @@ typedef struct Pre3_0alpha5Header {
     Aaddr ruleTableAddress;
     Aaddr messageTableAddress;
     /* Miscellaneous */
-    Aint attributesAreaSize;	/* Size of attribute data area in Awords */
-    Aint maxParameters;		/* Maximum number of parameters in any syntax */
-    Aaddr stringInitTable;	/* String init table address */
-    Aaddr setInitTable;		/* Set init table address */
-    Aaddr start;		/* Address to Start code */
-    Aword maximumScore;		/* Maximum score */
-    Aaddr scores;		/* Score table */
-    Aint scoreCount;		/* Max index into scores table */
-    Aaddr sourceFileTable;	/* Table of fpos/len for source filenames */
-    Aaddr sourceLineTable;	/* Table of available source lines to break on */
-    Aaddr freq;			/* Address to Char freq's for coding */
-    Aword acdcrc;		/* Checksum for acd code (excl. hdr) */
-    Aword txtcrc;		/* Checksum for text data file */
+    Aint attributesAreaSize;    /* Size of attribute data area in Awords */
+    Aint maxParameters;         /* Maximum number of parameters in any syntax */
+    Aaddr stringInitTable;      /* String init table address */
+    Aaddr setInitTable;         /* Set init table address */
+    Aaddr start;                /* Address to Start code */
+    Aword maximumScore;         /* Maximum score */
+    Aaddr scores;               /* Score table */
+    Aint scoreCount;            /* Max index into scores table */
+    Aaddr sourceFileTable;      /* Table of fpos/len for source filenames */
+    Aaddr sourceLineTable;      /* Table of available source lines to break on */
+    Aaddr freq;                 /* Address to Char freq's for coding */
+    Aword acdcrc;               /* Checksum for acd code (excl. hdr) */
+    Aword txtcrc;               /* Checksum for text data file */
 } Pre3_0alpha5Header;
 
 /* Error message numbers */
