@@ -66,9 +66,6 @@ static void showAttributes(AttributeEntry *attributes)
     i = 1;
     for (at = attributes; !isEndOfArray(at); at++) {
         sprintf(str, "$i$t%s[%d] = %d", (char *) pointerTo(at->id), at->code, (int)at->value);
-#if ISO == 0
-        fromIso(str, str);
-#endif
         output(str);
         i++;
     }
@@ -80,13 +77,13 @@ static void showContents(int cnt)
 {
     int i;
     char str[80];
-    Abool found = FALSE;
+    Abool found = false;
 
     output("$iContains:");
     for (i = 1; i <= header->instanceMax; i++) {
         if (isIn(i, cnt, DIRECT)) { /* Yes, it's directly in this container */
             if (!found)
-                found = TRUE;
+                found = true;
             output("$i$t");
             say(i);
             sprintf(str, "[%d] ", i);
@@ -153,13 +150,13 @@ static void listInstance(int ins) {
 static void listInstances(char *pattern)
 {
     int ins;
-    bool found = FALSE;
+    bool found = false;
 
     for (ins = 1; ins <= header->instanceMax; ins++) {
         if (pattern == NULL || (pattern != NULL && match(pattern, idOfInstance(ins)))) {
             if (!found) {
                 output("Instances:");
-                found = TRUE;
+                found = true;
             }
             listInstance(ins);
         }
@@ -189,7 +186,7 @@ static void showInstance(int ins)
     if (!isA(ins, header->locationClassId) || (isA(ins, header->locationClassId) && admin[ins].location != 0)) {
         sprintf(str, "$iLocation:");
         output(str);
-        needSpace = TRUE;
+        needSpace = true;
         sayLocationOfInstance(ins, NULL);
     }
 
@@ -271,7 +268,7 @@ static void showContainer(int cnt)
     if (containers[cnt].owner != 0) {
         cnt = containers[cnt].owner;
         say(cnt);
-        sprintf(str, "$iLocation: %d", where(cnt, TRUE));
+        sprintf(str, "$iLocation: %d", where(cnt, true));
         output(str);
     }
     showContents(cnt);
@@ -435,11 +432,8 @@ static void showEvents(void)
     output("Events:");
     for (event = 1; event <= header->eventMax; event++) {
         sprintf(str, "$i%d [%s]:", event, (char *)pointerTo(events[event].id));
-#if ISO == 0
-        fromIso(str, str);
-#endif
         output(str);
-        scheduled = FALSE;
+        scheduled = false;
         for (i = 0; i < eventQueueTop; i++)
             if ((scheduled = (eventQueue[i].event == event)))
                 break;
@@ -543,13 +537,13 @@ static int findSourceLineIndex(SourceLineEntry *entry, int file, int line) {
 /*----------------------------------------------------------------------*/
 static void listBreakpoints() {
     int i;
-    bool found = FALSE;
+    bool found = false;
 
     for (i = 0; i < BREAKPOINTMAX; i++)
         if (breakpoint[i].line != 0) {
             if (!found)
                 printf("Breakpoints set:\n");
-            found = TRUE;
+            found = true;
             printf("    %s:%d\n", sourceFileName(breakpoint[i].file), breakpoint[i].line);
         }
     if (!found)
@@ -630,12 +624,12 @@ static int loc;
 void saveInfo(void)
 {
     /* Save some important things */
-    saved_capitilize = capitalize; capitalize = FALSE;
-    saved_traceSection = traceSectionOption; traceSectionOption = FALSE;
-    saved_traceSource = traceSourceOption; traceSourceOption = FALSE;
-    saved_traceInstruction = traceInstructionOption; traceInstructionOption = FALSE;
-    saved_tracePush = tracePushOption; tracePushOption = FALSE;
-    saved_traceStack = traceStackOption; traceStackOption = FALSE;
+    saved_capitilize = capitalize; capitalize = false;
+    saved_traceSection = traceSectionOption; traceSectionOption = false;
+    saved_traceSource = traceSourceOption; traceSourceOption = false;
+    saved_traceInstruction = traceInstructionOption; traceInstructionOption = false;
+    saved_tracePush = tracePushOption; tracePushOption = false;
+    saved_traceStack = traceStackOption; traceStackOption = false;
     loc = current.location; current.location = where(HERO, DIRECT);
 }
 
@@ -699,7 +693,7 @@ static DebugParseEntry commandEntries[] = {
     {"actors", "[n]", ACTORS_COMMAND, "list instance(s) that are actors"},
     {"locations", "[n]", LOCATIONS_COMMAND, "list instances that are locations"},
     {"trace", "('source'|'section'|'instruction'|'push'|'stack')", TRACE_COMMAND, "toggle various traces"},
-    {"next", "", NEXT_COMMAND, "run game and stop at next source line"},
+    {"next", "", NEXT_COMMAND, "continue game and stop at next source line"},
     {"go", "", GO_COMMAND, "go another player turn"},
     {"exit", "", EXIT_COMMAND, "exit to game, enter 'debug' to get back"},
     {"x", "", EXIT_COMMAND, "d:o"},
@@ -787,7 +781,7 @@ static char parseDebugCommand(char *command) {
 static void readCommand(char buf[]) {
     char c;
 
-    capitalize = FALSE;
+    capitalize = false;
     if (anyOutput) newline();
     do {
         output("adbg> ");
@@ -817,7 +811,7 @@ static void displaySourceLocation(int line, int fileNumber) {
     printf("%s %s %s:%d\n", debugPrefix, cause, sourceFileName(fileNumber), line);
     showSourceLine(fileNumber, line);
     printf("\n");
-    anyOutput = FALSE;
+    anyOutput = false;
 }
 
 
@@ -832,9 +826,9 @@ static void toggleSectionTrace() {
 /*----------------------------------------------------------------------*/
 static void toggleInstructionTrace() {
     if ((saved_traceInstruction = !saved_traceInstruction))
-        printf("Single instruction trace on.");
+        printf("Instruction trace on.");
     else
-        printf("Single instruction trace off.");
+        printf("Instruction trace off.");
 }
 
 /*----------------------------------------------------------------------*/
@@ -951,8 +945,8 @@ static void handleDeleteCommand(bool calledFromBreakpoint, int line, int fileNum
 
 /*----------------------------------------------------------------------*/
 static void handleNextCommand(bool calledFromBreakpoint) {
-    stopAtNextLine = TRUE;
-    debugOption = FALSE;
+    stopAtNextLine = true;
+    debugOption = false;
     if (!calledFromBreakpoint)
         current.sourceLine = 0;
     restoreInfo();
@@ -1035,7 +1029,7 @@ static bool exactSameVersion() {
 /*======================================================================*/
 void debug(bool calledFromBreakpoint, int line, int fileNumber)
 {
-    static bool warned = FALSE;
+    static bool warned = false;
 
     saveInfo();
 
@@ -1051,11 +1045,11 @@ void debug(bool calledFromBreakpoint, int line, int fileNumber)
                    decodedGameVersion(header->version));
             printf("<That is not exactly the same as this interpreter (%s).>\n", alan.version.string);
             printf("<This might cause a lot of trouble. Cross your fingers...>\n");
-            warned = TRUE;
+            warned = true;
         }
     }
 
-    while (TRUE) {
+    while (true) {
 
         char commandLine[200];
         readCommand(commandLine);
@@ -1069,7 +1063,7 @@ void debug(bool calledFromBreakpoint, int line, int fileNumber)
         case CLASSES_COMMAND: handleClassesCommand(); break;
         case DELETE_COMMAND: handleDeleteCommand(calledFromBreakpoint, line, fileNumber); break;
         case EVENTS_COMMAND: showEvents(); break;
-        case EXIT_COMMAND: debugOption = FALSE; restoreInfo(); goto exit_debug;
+        case EXIT_COMMAND: debugOption = false; restoreInfo(); goto exit_debug;
         case FILES_COMMAND: listFiles(); break;
         case LINES_COMMAND: listLines(); break;
         case GO_COMMAND: restoreInfo(); goto exit_debug;
@@ -1094,23 +1088,23 @@ void debug(bool calledFromBreakpoint, int line, int fileNumber)
 }
 
 
+/*
+  Say something, but make sure we don't disturb anything and that it is
+  shown to the player. Needed for tracing. During debugging things are
+  set up to avoid this problem.
+*/
+
 /*======================================================================*/
 void traceSay(int item)
 {
-    /*
-      Say something, but make sure we don't disturb anything and that it is
-      shown to the player. Needed for tracing. During debugging things are
-      set up to avoid this problem.
-    */
-
     saveInfo();
-    needSpace = FALSE;
+    needSpace = false;
     col = 1;
     if (item == 0)
         printf("$null$");
     else
         say(item);
-    needSpace = FALSE;
+    needSpace = false;
     col = 1;
     restoreInfo();
 }
