@@ -45,15 +45,12 @@
 #include "glk.h"
 #include "glkio.h"
 #ifdef HAVE_WINGLK
-#include "winglk.h"
+#include "WinGlk.h"
 #endif
 #endif
 
 /* PUBLIC DATA */
 
-winid_t glkMainWin;
-winid_t glkStatusWin;
-
 /* Amachine structures - Static */
 VerbEntry *vrbs;		/* Verb table pointer */
 
@@ -187,11 +184,11 @@ static void readTemporaryHeader(ACodeHeader *tmphdr) {
 /*----------------------------------------------------------------------*/
 static void reverseMemory() {
     if (littleEndian()) {
-        if (debugOption||traceSectionOption||traceInstructionOption)
-            output("<Hmm, this is a little-endian machine, fixing byte ordering....");
+        if ((debugOption||traceSectionOption||traceInstructionOption) && !regressionTestOption)
+            printf("<Hmm, this is a little-endian machine, fixing byte ordering....");
         reverseACD();			/* Reverse content of the ACD file */
-        if (debugOption||traceSectionOption||traceInstructionOption)
-            output("OK.>$n");
+        if ((debugOption||traceSectionOption||traceInstructionOption) && !regressionTestOption)
+            printf("OK.>\n");
     }
 }
 
@@ -240,9 +237,7 @@ static void loadAndCheckMemory(ACodeHeader tmphdr, Aword crc, char err[]) {
         if (!ignoreErrorOption)
             syserr(err);
         else {
-            output("<WARNING! $$");
-            output(err);
-            output("$$ Ignored, proceed at your own risk.>$n");
+            printf("%s%s%s", "<WARNING! ", err, " - Ignored, proceed at your own risk.>\n");
         }
     }
 }
@@ -303,12 +298,12 @@ static void incompatibleVersion(ACodeHeader *header) {
 
 /*----------------------------------------------------------------------*/
 static void alphaRunningLaterGame(char gameState) {
-    output("<WARNING! You are running an alpha interpreter, but the game is generated by a");
+    printf("<WARNING! You are running an alpha interpreter, but the game is generated by a ");
     if (gameState == 'b')
-        output("beta");
+        printf("beta");
     else
-        output("release");
-    output("state compiler which was released later. This might cause the game to not work fully as intended. Look for an upgraded game file.>\n");
+        printf("release");
+    printf(" state compiler which was released later. This might cause the game to not work fully as intended. Look for an upgraded game file.>\n");
 }
 
 /*----------------------------------------------------------------------*/
@@ -384,7 +379,7 @@ void checkVersion(ACodeHeader *header)
                 else
                     incompatibleVersion(header);
             } else
-                output("<WARNING! Incompatible version of ACODE program.>\n");
+                printf("<WARNING! Incompatible version of ACODE program.>\n");
         } else if (developmentVersion && gameState != 'd')
             /* ... unless interpreter is development and game not */
             incompatibleDevelopmentVersion(header);
@@ -430,10 +425,10 @@ static void checkDebug(void)
             printf("<Sorry, '%s' is not compiled for debug! Exiting.>\n", adventureFileName);
             terminate(0);
         }
-        debugOption = FALSE;
-        traceSectionOption = FALSE;
-        traceInstructionOption = FALSE;
-        tracePushOption = FALSE;
+        debugOption = false;
+        traceSectionOption = false;
+        traceInstructionOption = false;
+        tracePushOption = false;
     }
 
     if (debugOption || regressionTestOption) /* If debugging or regression testing... */
@@ -626,8 +621,10 @@ static void start(Stack theStack)
     para();
 
     if (where(HERO, TRANSITIVE) == startloc) {
-        if (traceSectionOption)
-            printf("<CURRENT LOCATION:>");
+        if (traceSectionOption) {
+            printf("<CURRENT LOCATION:>\n");
+            anyOutput = false;
+        }
         look();
     }
     resetAndEvaluateRules(rules, header->version, theStack);
@@ -657,10 +654,13 @@ static void openFiles(void)
         apperr(str);
     }
 
-    /* If logging open log file */
-    if (transcriptOption || logOption) {
+    /* If logging open transcript and/or log file */
+    if (transcriptOption) {
         startTranscript();
     }
+    if (commandLogOption) {
+        startCommandLog();
+    }
 }
 
 
@@ -687,7 +687,7 @@ static void init(Stack theStack)
 
     /* Start the adventure */
     if (debugOption)
-        debug(FALSE, 0, 0);
+        debug(false, 0, 0);
     else
         clear();
 
@@ -741,8 +741,8 @@ static void moveActor(int theActor)
         /* Ask him! */
         if (setjmp(forfeitLabel) == 0) {
             parse();
-            capitalize = TRUE;
-            fail = FALSE;			/* fail only aborts one actor */
+            capitalize = true;
+            fail = false;			/* fail only aborts one actor */
         }
     } else if (admin[theActor].script != 0) {
         for (scr = (ScriptEntry *) pointerTo(header->scriptTableAddress); !isEndOfArray(scr); scr++) {
@@ -785,7 +785,7 @@ static void moveActor(int theActor)
                 if (fail || (admin[theActor].step != 0 && isEndOfArray(step)))
                     /* No more steps in this script, so stop him */
                     admin[theActor].script = 0;
-                fail = FALSE;			/* fail only aborts one actor */
+                fail = false;			/* fail only aborts one actor */
                 break;			/* We have executed a script so leave loop */
             }
         }
@@ -833,9 +833,9 @@ void run(void)
     if (!ERROR_RETURNED)      /* Can happen in start section too... */
         init(theStack);               /* Initialise and start the adventure */
 
-    while (TRUE) {
+    while (true) {
         if (debugOption)
-            debug(FALSE, 0, 0);
+            debug(false, 0, 0);
 
         if (stackDepth(theStack) != 0)
             syserr("Stack is not empty in main loop");
@@ -862,7 +862,7 @@ void run(void)
 
         /* Move all characters, hero first */
         rememberGameState();
-        current.meta = FALSE;
+        current.meta = false;
         moveActor(header->theHero);
 
         if (gameStateChanged)
