@@ -1,5 +1,5 @@
-#ifndef UTILS_H_
-#define UTILS_H_
+#ifndef _UTILS_H_
+#define _UTILS_H_
 /*----------------------------------------------------------------------*\
 
     utils.h
@@ -22,4 +22,4 @@ extern void printVersion(int buildNumber);
 extern void printIFIDs(char *adventureName);
 extern bool match(char *pattern, char *input);
 
-#endif /* UTILS_H_ */
+#endif
