@@ -1,14 +1,13 @@
-// vim: set ft=c:
+// vim: set ft=cpp:
 
 #ifndef ZTERP_DICT_H
 #define ZTERP_DICT_H
 
-#include <stdint.h>
-#include <stdbool.h>
+#include "types.h"
 
 void tokenize(uint16_t text, uint16_t parse, uint16_t dictaddr, bool flag);
 
-void ztokenise(void);
-void zencode_text(void);
+void ztokenise();
+void zencode_text();
 
 #endif
