@@ -28,6 +28,10 @@
 #include "os.h"
 #include "glk.h"
 
+#ifdef GARGLK
+#include "glkstart.h"
+#endif
+
 /* for version strings */
 #include "trd.h"
 #include "vmvsn.h"
@@ -540,7 +544,9 @@ int os_askfile(const char *prompt, char *fname_buf, int fname_buf_len,
     if (fileref == NULL)
         return OS_AFE_CANCEL;
 
-    strcpy(fname_buf, garglk_fileref_get_name(fileref));
+#ifdef GLKUNIX_FILEREF_GET_FILENAME
+    strcpy(fname_buf, glkunix_fileref_get_filename(fileref));
+#endif
 
     glk_fileref_destroy(fileref);
 
